@@ -29,7 +29,19 @@ class Client:
 
     async def _setup(self) -> None:
         try:
-            self._reader, self._writer = await asyncio.wait_for(asyncio.open_connection(self.host, self.port, loop=self._loop), timeout=self.timeout, loop=self._loop)
+            result = await asyncio.gather(
+                asyncio.wait_for(
+                    asyncio.open_connection(self.host, self.port, loop=self._loop), timeout=self.timeout, loop=self._loop
+                ),
+                loop=self._loop, return_exceptions=True
+            )
+
+            result = result[0]
+
+            if isinstance(result, Exception):
+                raise result
+
+            self._reader, self._writer = result
         except TimeoutError:
             self._closed = True
             raise ConnectionFailedError('A timeout occurred while attempting to connect to the server')
@@ -58,6 +70,7 @@ class Client:
         out_msg = struct.pack('<li', 0, _type) + msg.encode('utf8') + b'\x00\x00'
         out_len = struct.pack('<i', len(out_msg))
         self._writer.write(out_len + out_msg)
+        await self._writer.drain()
 
         in_msg = await self._read(struct.unpack('<i', await self._read(4))[0])
         if in_msg[-2:] != b'\x00\x00': raise InvalidDataReceivedError
